Package: ps
Title: List, Query, Manipulate System Processes
Version: 1.8.1
Authors@R: c(
    person("Jay", "Loden", role = "aut"),
    person("Dave", "Daeschler", role = "aut"),
    person("Giampaolo", "Rodola'", role = "aut"),
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: List, query and manipulate all system processes, on
    'Windows', 'Linux' and 'macOS'.
License: MIT + file LICENSE
URL: https://github.com/r-lib/ps, https://ps.r-lib.org/
BugReports: https://github.com/r-lib/ps/issues
Depends: R (>= 3.4)
Imports: utils
Suggests: callr, covr, curl, pillar, pingr, processx (>= 3.1.0), R6,
        rlang, testthat (>= 3.0.0), webfakes, withr
Biarch: true
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Author: Jay Loden [aut],
  Dave Daeschler [aut],
  Giampaolo Rodola' [aut],
  Gábor Csárdi [aut, cre],
  Posit Software, PBC [cph, fnd]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Built: R 4.5.1; aarch64-apple-darwin25.0.0; 2025-09-17 14:49:48 UTC; unix
